class Programs:
    """
    A class defining all existing programs
    Also defines the maximum number of available programs
    """

    PGM_1 = 1
    PGM_2 = 2
    PGM_3 = 3
    PGM_4 = 4

    PGM_MAX = 4
